"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A processor auditing plugin that should work with most Redhat-like distros

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

class ProcessorPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):
        processor_info = {}

        cpu_index = None
        with open('/proc/cpuinfo') as fp:
            for line in fp.read().splitlines():
                name_value = map(str.strip, str.split(line, ':', 1))
                if len(name_value) != 2:
                    continue
                name, value = name_value
                if name == 'processor':
                    cpu_index = value
                    if cpu_index not in processor_info:
                        processor_info[cpu_index] = {}

                if cpu_index is None:
                    continue

                if name == 'cpu MHz':
                    processor_info[cpu_index]['mhz'] = value
                elif name == 'model name':
                    processor_info[cpu_index]['identifier'] = value
                    processor_info[cpu_index]['name_string'] = value
                elif name == 'vendor_id':
                    processor_info[cpu_index]['vendor_identifier'] = value

        return processor_info




"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A memory auditing plugin that should work on most Redhat-like distros

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

class MemoryPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):
        memory_info = {}

        with open('/proc/meminfo') as fp:
            for line in fp.read().splitlines():
                name_value = map(str.strip, str.split(line, ':', 1))
                if len(name_value) != 2:
                    continue
                name, value = name_value

                if name == 'MemFree':
                    memory_info['available_bytes'] = long(value.split()[0])
                elif name == 'MemTotal':
                    memory_info['total_bytes'] = long(value.split()[0])
                if name == 'SwapFree':
                    memory_info['available_swap_bytes'] = long(value.split()[0])
                elif name == 'SwapTotal':
                    memory_info['total_swap_bytes'] = long(value.split()[0])
                if name == 'VmallocFree':
                    memory_info['available_virtual_bytes'] = long(value.split()[0])
                if name == 'VmallocTotal':
                    memory_info['total_virtual_bytes'] = long(value.split()[0])

        return memory_info




"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A installed packages auditing plugin that should work with CentOS

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
"""
import abc
from subprocess import Popen, PIPE
import shlex
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

class PackageManagerBase(object):
    """
    Base class for all the different linux package managers
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def list_packages(self):
        """
        @return: A dictionary of package name keys and package version values
        @rtype: C{dict(str: str)}
        """
        pass

class PackageManagerDpkg(PackageManagerBase):
    """
    Gets listing of packages and versions using dpkg
    """
    def list_packages(self):
        software = {}

        pkg_list_cmd = '/usr/bin/dpkg -l'
        pkg_list_out = Popen(shlex.split(pkg_list_cmd), stdout=PIPE).stdout.read().splitlines()
        _logger.debug('pkg_list command results [%s]: %s', pkg_list_cmd, pkg_list_out)

        for line in pkg_list_out:
            cols = line.strip().split()
            if cols[0] == 'ii' and len(cols) >= 3:
                software[cols[1]] = cols[2]

        return software

class PackageManagerRpm(PackageManagerBase):
    """
    Gets listing of packages and versions using rpm
    """
    def list_packages(self):
        software = {}

        pkg_list_cmd = '/bin/rpm -qa --queryformat="%{NAME} %{VERSION}\\n"'
        pkg_list_out = Popen(shlex.split(pkg_list_cmd), stdout=PIPE).stdout.read().splitlines()
        _logger.debug('pkg_list command results [%s]: %s', pkg_list_cmd, pkg_list_out)

        for line in pkg_list_out:
            cols = line.strip().split()
            if len(cols) >= 2:
                software[cols[0]] = cols[1]

        return software




"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    pyHai base classes

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
import logging
import abc


# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())


class AuditorPlugin(object):
    """
    An ABC class to enforce a common plugin interface
    """
    __metaclass__ = abc.ABCMeta

    __profile = None
    __results = None
    _running_audit = None


    def __init__(self, profile, running_audit_results, *args, **kwargs):
        """
        You probably shouldn't overwrite this method unless you know what you're doing and
        even then you should be careful and call Plugin.__init__.py at some point.
        I{Swim at your own risk}
        
        @param profile: A dictionary containing this host's profile
        @type profile: C{dict}
        @param running_audit_results: A dictionary of the results of all the plugins that have run up to "now"
        @type running_audit_results: C{dict}
        @return: A dictionary of results from the plugin's run
        @rtype: C{dict}
        """
        self.__profile = profile
        self._running_audit = running_audit_results
        try:
            before_results = self.before(*args, **kwargs)
            self.__set_results(self.run(*args, before_results=before_results, **kwargs))
            self.__set_results(self.after(*args, **kwargs))
        except Exception as exc:
            self.fail(exc)

    @abc.abstractmethod
    def run(self, *args, **kwargs):
        """
        Will be called after the before hook.
        This is the only "required" method for plugins
        """
        pass

    def before(self, *args, **kwargs):
        """
        Overwrite in subclass as necessary
        Will be called first during init
        """
        _logger.debug('Plugin hook "before" is not overwritten')
        return None

    def after(self, *args, **kwargs):
        """
        Overwrite in subclass as necessary
        Will be called first during init
        """
        _logger.debug('Plugin hook "after" is not overwritten')
        return self._get_results()

    def fail(self, exc, msg=None, *args):
        """
        Overwrite in subclass as necessary
        Will be called for any exceptions raised for before, run, or after methods
        """
        _logger.debug('Current state of results in %s: %s',)
        _logger.exception(msg, *args)

    def get_profile(self, key=None):
        """
        Gets the entire profile dictionary, a list of items or a single key from the profile dictionary
        
        @param key: Providing a string returns a single value from the profile dictionary,
            providing an array, returns a slice of the profile dictionary, omitting this param
            returns the entire profile dictionary
        @type key: C{list | str}
        @return: Mixed... part or all of the profile dictionary
        @rtype: C{dict | str}
        """
        if key is not None:
            if type(key) is list:
                return dict([(k, self.__profile[k]) for k in key if k in self.__profile])
            elif type(key) is str and key in self.__profile:
                return self.__profile[key]
            else:
                return None
        else:
            return self.__profile

    def __set_results(self, results):
        """
        Ensures that only dict types are stored in the __results instance var
        
        @param results: The results from the plugin_hooks or the run
            B{**MUST} be a dict or will raise an exception 
        @type results: C{dict}
        """
        result_type = type(results)
        if result_type is dict or result_type is tuple or result_type is list:
            self.__results = results

    def _get_results(self):
        """
        Returns results
        
        @return: Results of the run
        @rtype: C{dict}
        """
        return self.__results



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A processor auditor that should work on most Windows hosts

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import sys
import logging

win32api = None
win32con = None

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

try:
    import win32api
    import win32con
except:
    _logger.exception('%s failed to import required Windows specific modules: %s', __name__, ', '.join('win32api', 'win32con'))
    raise

class ProcessorPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):
        processor_info = {}
        processor_description_key_path = 'HARDWARE\DESCRIPTION\System\CentralProcessor'
        system_env_vars_key = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE, processor_description_key_path)
        value_index = 0
        try:
            while True:
                cpu_index = win32api.RegEnumKey(system_env_vars_key, value_index)
                cpu_reg = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE, processor_description_key_path + '\\' + cpu_index)
                processor_info[cpu_index] = {
                    'mhz': win32api.RegQueryValueEx(cpu_reg, '~MHz')[0],
                    'identifier': win32api.RegQueryValueEx(cpu_reg, 'Identifier')[0],
                    'name_string': win32api.RegQueryValueEx(cpu_reg, 'ProcessorNameString')[0],
                    'vendor_identifier': win32api.RegQueryValueEx(cpu_reg, 'VendorIdentifier')[0],
                    }
                value_index += 1
        except:
            exc_id = sys.exc_info()[1][0]
            if exc_id != 259: # 259 == 'No more data is available.'
                raise

        processor_info['count'] = len(processor_info)

        return processor_info



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A default implementation of the profiler

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
import sys
import platform
import socket
from .base import ProfilerBase

class DefaultProfiler(ProfilerBase):
    """
    The default profiler plugin
    """
    __profile = None

    def __init__(self):
        """
        Initialize profiler
        """
        system_class = str(platform.system().lower())
        system = self.__detect_system(system_class)
        self.__profile = {
            'system_class': system_class,
            'system': system,
            'architecture': platform.architecture()[0],
            'python_version': platform.python_version(),
            'hostname': socket.gethostname(),
            'fqdn': socket.getfqdn(),
            }

    def profile(self):
        """
        Returns the system's profile
        
        @return: A dictionary consisting of the system and architecture
        @rtype: C{dict}
        """
        return self.__profile

    def system(self):
        """
        Returns the system property of the system's profile
        
        @return: The system property of the system's profile
        @rtype: C{str}
        """
        return self.__profile['system']

    def system_class(self):
        """
        Returns the system class property - windows, linux, java
        
        @return: The type of system
        @rtype: C{str}
        """
        return self.__profile['system_class']

    def __detect_system(self, system_class):
        """
        Detects the specific type of system. For windows, the windows type and version,
        for linux, the distribution and version
        
        @return: A string representing the specific type of system: windows7, windowsxp, ubuntu, centos, etc
        @rtype: C{str}
        """
        if system_class == 'linux':
            if hasattr(platform, 'linux_distribution'):
                distro = platform.linux_distribution()
            elif hasattr(platform, 'dist'):
                distro = platform.dist()
            else:
                distro = None
            if distro and type(distro) is tuple and len(distro) > 0:
                system = str(distro[0]).lower()
                system_parts = system.split()
                if system_parts[0] in ('centos'):
                    return system_parts[0]
                else:
                    return system
        elif system_class == 'windows':
            return str(platform.win32_ver()[0]).lower()
        else:
            return None



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A system profiler/auditor inspired by Opscode's Ohai

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
import os
import sys
import logging
from .utils import _underscore_to_camel_case
from .profilers.base import ProfilerBase
from .profilers.default import DefaultProfiler

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

# current version
__VERSION__ = (0, 1, 1)
VERSION = '.'.join(map(str, __VERSION__))

# set some default variables
PACKAGE_PLUGIN_PATH = os.path.join('pyhai', os.path.dirname(os.path.realpath(__file__)), 'plugins')
DEFAULT_CUSTOM_PLUGIN_PATH = os.path.join(os.path.dirname(os.path.realpath('.')), 'plugins')
PLUGIN_LOADER_EXCLUSIONS = ('.', '..', '__init__.py', '__init__.pyc', '__init__.pyo')
DEFAULT_PROFILER_CLASS = DefaultProfiler


class Auditor(object):
    """
    Auditor class
    
    @ivar custom_plugin_path: The path to any custom plugins
    @type custom_plugin_path: C{str}
    @ivar profile: A dictionary of properties for this host
    @type profile: C{dict}
    @ivar architecture: The name of the architecture as is normally returned by platform.architecture()[0]
    @type architecture: C{str}
    @ivar plugins: A list of successfully loaded plugins
    @type plugins: C{list}
    """
    plugin_paths = [PACKAGE_PLUGIN_PATH]
    profile = None
    plugins = {}

    def __init__(self, plugin_paths=None, **kwargs):
        """
        Initialize System object
        
        @param plugin_paths: A path (or list of paths) to a custom set of plugins
        @type plugin_paths: C{str | list}
        @keyword profiler_class: The name of a class that extends L{ProfilerBase} or the name of the module where the
            L{ProfilerBase} class can be found. If supplying a module, must supply the profiler_class keyword arg
        @type profiler_class: C{class | str}
        @keyword enable_default_plugins: A flag to use (or suppress) the builtin plugins
        @type enable_default_plugins: C{bool}
        @keyword profiler_package: The name of the package that contains a class that extends L{ProfilerBase}
        @type profiler_package: C{str} 
        """
        profiler_class = kwargs.get('profiler_class', DEFAULT_PROFILER_CLASS)
        profiler_package = kwargs.get('profiler_package', None)
        enable_default_plugins = kwargs.get('enable_default_plugins', True)


        if type(profiler_class) is str:
            profiler_package = kwargs.get('profiler_package', '')
            if profiler_package:
                profiler_class = self.__load_module(profiler_package, profiler_class)
            else:
                profiler_class = self.__load_module(profiler_class)

        if issubclass(profiler_class, ProfilerBase):
            profiler = profiler_class()
            if hasattr(profiler, 'profile'):
                self.profile = profiler.profile()
                if type(self.profile) is dict and 'pyhai_version' not in self.profile:
                    self.profile['pyhai_version'] = VERSION
                self.system_class = profiler.system_class()
                self.system = profiler.system()
                _logger.debug('Successfully loaded the profiler: %s', profiler.__class__.__name__)
            else:
                raise Exception('Failed to load a valid profiler: %s' % profiler.__class__.__name__)
        else:
            raise Exception('Arguments supplied for profiler are not valid')

        if plugin_paths:
            if type(plugin_paths) is str:
                plugin_paths = [plugin_paths]

            if enable_default_plugins:
                for path in plugin_paths:
                    if path not in self.plugin_paths:
                        self.plugin_paths.append(path)
            else:
                self.plugin_paths = plugin_paths
        elif not plugin_paths and not enable_default_plugins:
            # TODO: Is there a better way to do this? Would like to stop execution and allow for exception information to
            #    be sent to the logger.
            try:
                raise ValueError('Incompatible init params... plugin_paths is empty and enable_default_plugins=False')
            except:
                _logger.exception('Must provide a list for plugin_paths or set enable_default_plugins=True. Nothing to do.')
                raise

        for path in self.plugin_paths:
            sys.path.insert(0, path)
        _logger.debug('Setting plugin_paths to: ["%s"]', '", "'.join(self.plugin_paths))
        self.__load_plugins(self.system_class, self.system)


    def __load_profiler(self, profiler, package=''):
        """
        Loads a profiler plugin
        
        @param profiler: The name of a class that extends L{ProfilerBase}
        @type profiler: C{str}
        @param package: The name of the package where the profiler exists
        @type package: C{str}
        """
        return self.__load_module(profiler, package)


    def __load_plugins(self, system_class, system, plugin_module=None):
        """
        Imports plugin modules and stores the list of successfully loaded plugins
        
        @param plugin_module: A specific plugin_module to load
        @type plugin_module: C{str}
        """
        _logger.info('Loading plugins...')
        if plugin_module:
            raise NotImplementedError('Planned for future release')
        else:
            plugin_map = self.__resolve_plugin_paths(system_class, system)
            for plugin, (path, namespace) in plugin_map.items():
                _logger.debug('Loading plugin: %s, path: %s, package: %s', plugin, path, namespace)
                if path not in sys.path:
                    sys.path.insert(0, path)
                try:
                    plugin_class = '%s%s' % (_underscore_to_camel_case(plugin), 'Plugin')
                    self.plugins[plugin] = self.__load_module(namespace, plugin_class)
                    _logger.debug('Loaded plugin: %s [%s::%s]', plugin, namespace, plugin_class)
                except:
                    _logger.exception('Failed to load plugin: %s [%s::%s]', plugin, namespace, plugin_class)


    def __load_module(self, module, cls=''):
        """
        Loads a module and class dynamically return a reference to the class
        
        @param module: The name of module to load
        @type module: C{str}
        @param cls: The name of a class to load
        @type cls: C{str}
        @return: A reference to the loaded class
        @rtype: C{class}
        """
        try:
            module_instance = __import__(module, globals(), locals(), [cls])
            if cls and hasattr(module_instance, cls):
                _logger.debug('Successfully loaded module: %s, class: %s', module, cls)
                return getattr(module_instance, cls)
        except:
            _logger.exception('Failed to import module: %s, class: %s', module, cls)
            raise


    def __resolve_plugin_paths(self, system_class, system, **kwargs):
        """
        Checks plugin paths for validity and returns only those that are valid

        @param system: The type of system
        @type system: C{str}
        @keyword plugin_paths: A path (or list of paths) to a custom set of plugins
        @type plugin_paths: C{str}|C{list}
        @return: A list of valid plugin paths
        @rtype: C{list}
        """
        plugins = {}
        plugin_paths = kwargs.get('plugin_paths', self.plugin_paths)
        for plugin_path in plugin_paths:
            _logger.debug('Searching plugin path: %s', plugin_path)
            system_class_path = os.path.join(plugin_path, system_class)
            system_path = os.path.join(system_class_path, system)

            valid_plugins = self.__validate_plugins(plugin_path)
            valid_plugins = self.__validate_plugins(system_class_path, system_class, valid_plugins)
            valid_plugins = self.__validate_plugins(system_path, '%s.%s' % (system_class, system), valid_plugins)

            if valid_plugins is not None and len(valid_plugins) > 0:
                _logger.debug('Merging plugins dictionaries')
                plugins = dict(plugins.items() + valid_plugins.items())

        return plugins


    def __validate_plugins(self, path, base=None, plugins=None):
        """
        Performs an initial sanity check on all the plugins found in a path
        
        @param path: The path to look for plugins
        @type path: C{str}
        @param base: The base of the package name if path is a subfolder of a python package
            - I{To assist with the import, i.e. from package.module import plugin}
        @type base: C{str}
        """
        if plugins is None:
            _logger.debug('Creating plugins dict')
            plugins = {}

        if os.path.exists(path) and os.path.isdir(path):
            for plugin_entry in os.listdir(path):
                _logger.debug('Validating file as plugin: %s', plugin_entry)
                if plugin_entry in PLUGIN_LOADER_EXCLUSIONS or os.path.isdir(plugin_entry) or not plugin_entry.endswith('.py') or plugin_entry.startswith('_'):
                    continue

                plugin = plugin_entry[:-3]
                package = plugin

                if base is not None:
                    package = '%s.%s' % (base, plugin)
                    _logger.debug('package: %s', package)

                _logger.debug('File appears to be a valid plugin: %s, package: %s [%s]', plugin, package, os.path.join(path, plugin_entry))

                plugins[plugin] = (path, package)

        return plugins


    def audit(self):
        """
        Profiles the system using the default plugins and all custom plugins, returning a dictionary of the results
        
        @return: A dictionary representing the current state of the system
        @rtype: C{dict}
        """
        if len(self.plugins) <= 0:
            # TODO: Same deal as TODO in class __init__...
            try:
                raise ValueError('No plugins found')
            except:
                _logger.exception('List of plugins is empty. Nothing to do.')
                raise

        results = {'profile': self.profile}
        for plugin_name, plugin_class in self.plugins.items():
            _logger.debug('Running plugin: %s [%s]', plugin_name, plugin_class.__name__)
            plugin = plugin_class(self.profile, results)
            results[plugin_name] = plugin._get_results()
        return results



def audit(plugin_paths=DEFAULT_CUSTOM_PLUGIN_PATH, **kwargs):
    """
    Instatiates a System object and executes it's profile method
    
    @param plugin_paths: A path (or list of paths) to a custom set of plugins
    @type plugin_paths: C{str}|C{list}
    @keyword debug: Set the logging level to DEBUG
    @type debug: C{bool}
    """
    debug = kwargs.pop('debug', False)

    if debug:
        _logger.setLevel(logging.DEBUG)

    auditor = Auditor(plugin_paths, **kwargs)
    return auditor.audit()



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A network auditing plugin that should work on most Windows systems

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

try:
    from win32com.client import GetObject
except:
    _logger.exception('%s failed to import required Windows specific modules: %s', __name__, ', '.join('win32com.client.GetObject',))
    raise

class NetworkPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):
        network_info = {}

        wmi = GetObject(r'winmgmts:{impersonationLevel=impersonate}!\\.\root\cimv2')
        network_adaptor_configurations = wmi.ExecQuery('SELECT * FROM Win32_NetworkAdapterConfiguration WHERE IPEnabled = TRUE')
        network_adaptors = wmi.ExecQuery('SELECT * FROM Win32_NetworkAdapter')

        for nic_config in network_adaptor_configurations:
            if not nic_config.IPEnabled:
                continue

            config_index = str(nic_config.Index)
            if config_index not in network_info:
                network_info[config_index] = {}

            network_info[config_index]['dns_servers'] = [str(host) for host in self._get_or_list(nic_config.DNSServerSearchOrder)]
            network_info[config_index]['dhcp_enabled'] = nic_config.DHCPEnabled
            network_info[config_index]['dhcp_server'] = nic_config.DHCPServer
            network_info[config_index]['ip_address'] = [str(ip) for ip in self._get_or_list(nic_config.IPAddress)]
            network_info[config_index]['subnet_mask'] = [str(sm) for sm in self._get_or_list(nic_config.IPSubnet)]
            network_info[config_index]['default_gateway'] = [str(dg) for dg in self._get_or_list(nic_config.DefaultIPGateway)]
            network_info[config_index]['mac_address'] = nic_config.MACAddress
            network_info[config_index]['wins_servers'] = [str(nic_config.WINSPrimaryServer), str(nic_config.WINSSecondaryServer)]

        for nic in network_adaptors:
            nic_index = str(nic.Index)
            if nic_index not in network_info:
                continue

            network_info[nic_index]['name'] = nic.Name
            network_info[nic_index]['label'] = nic.NetConnectionID
            network_info[nic_index]['manufacturer'] = nic.Manufacturer
            network_info[nic_index]['device_id'] = nic.PNPDeviceID
            network_info[nic_index]['guid'] = nic.GUID
            network_info[nic_index]['speed'] = nic.Speed

        return network_info

    def _get_or_list(self, obj):
        try:
            if obj:
                return obj
            else:
                return list()
        except:
            return list()



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    pyHai profiler base class

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""

import abc

class ProfilerBase(object):
    """
    An ABC class to enforce a common profiler interface
    """
    __metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def profile(self):
        """
        Profile's the local system
        
        @return: A dictionary of properties discovered about the local system
        @rtype: C{dict}
        """
        pass

    @abc.abstractmethod
    def system(self):
        """
        Returns the system property for use in resolving plugins
        
        @return: The name of the current type of system
        @rtype: C{str}
        """
        pass

    @abc.abstractmethod
    def system_class(self):
        """
        Returns the system class property for use in resolving plugins
        
        @return: The name of the current type of system class
        @rtype: C{str}
        """
        pass



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A network auditing plugin that should work with most Redhat-like distros

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
from subprocess import Popen, PIPE
import os
import shlex
import platform
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())


class NetworkPlugin(AuditorPlugin):

    def run(self, *args, **kwargs):
        network_info = {}

        ifconfig_cmd = '/bin/env ifconfig -a'
        ifconfig_out = Popen(shlex.split(ifconfig_cmd), stdout=PIPE).stdout.read().splitlines()
        _logger.debug('ifconfig command results [%s]: %s', ifconfig_cmd, ifconfig_out)

        line_no = 0
        found_break = False
        nic = None
        for line in ifconfig_out:

            stripped_row = line.strip()
            if stripped_row == '':
                found_break = True
                continue

            row = stripped_row.split()

            if line_no == 0 or found_break:
                if found_break: found_break = False

                nic = row[0]

                if nic == 'lo':
                    nic = None
                    continue

                if nic not in network_info:
                    network_info[nic] = {}

                network_info[nic]['name'] = nic
                network_info[nic]['label'] = nic
                network_info[nic]['device_id'] = nic
                network_info[nic]['mac_address'] = row[4]

            if nic is not None:
                if row[0] == 'inet' and row[1].startswith('addr:'):
                    network_info[nic]['ip_address'] = row[1].split(':', 1)[1]
                    network_info[nic]['subnet_mask'] = row[3].split(':', 1)[1]

            line_no += 1


        try:
            distribution = platform.linux_distribution()[0]
        except:
            _logger.exception('Could not determine distribution.')
            return network_info

        dns_servers = []
        redhat_servers = ('CentOS Linux', 'Red Hat Enterprise Linux Server')
        if distribution in redhat_servers:
            dns_servers = self._redhat_dns_servers()

        for nic in network_info.keys():

            if distribution in redhat_servers:
                dhcp_enabled, dhcp_server = self._redhat_dhcp(nic)
                network_info[nic]['dhcp_enabled'] = dhcp_enabled
                network_info[nic]['dhcp_server'] = dhcp_server

                network_info[nic]['default_gateway'] = self._redhat_default_gateway()
                network_info[nic]['dns_servers'] = dns_servers
                network_info[nic]['speed'] = self._redhat_speed(nic)
                network_info[nic]['name'] = self._redhat_nic_name(nic)

        return network_info

    def _redhat_dhcp(self, nic):
        dhcp_enabled = False
        dhcp_servers = []
        dhcp_lease_report = '/var/lib/dhclient/dhclient-%s.leases' % nic
        if os.path.exists(dhcp_lease_report):
            with open(dhcp_lease_report) as fp:
                for line in fp.read().splitlines():
                    row = line.strip().split()
                    if len(row) >= 3:
                        if row[1] == 'dhcp-server-identifier' and row[2] not in dhcp_servers:
                            dhcp_servers.append(row[2])

        if dhcp_servers >= 1:
            dhcp_enabled = True
        return (dhcp_enabled, dhcp_servers)

    def _redhat_default_gateway(self):
        route_cmd = 'route -n'
        route_out = Popen(shlex.split(route_cmd), stdout=PIPE).stdout.read().splitlines()
        _logger.debug('route comand output [%s]: %s', route_cmd, route_out)

        for line in route_out:
            row = line.strip().split()
            if row[3] in ('UG', 'G'):
                return [row[1], ]

        return []


    def _redhat_dns_servers(self):
        dns_servers = []
        with open('/etc/resolv.conf') as fp:
            for line in fp.read().splitlines():
                row = line.strip().split()
                if row[0] == 'nameserver':
                    if row[1] not in dns_servers:
                        dns_servers.append(row[1])
        return dns_servers

    def _redhat_nic_name(self, nic):
        with open('/var/log/dmesg') as fp:
            for line in fp.read().splitlines():
                row = map(lambda s: s.strip(':'), line.strip().split())
                if len(row) <= 1:
                    continue
                if row[0] == nic or row[0] == nic:
                    return row[1]
                elif row[1] == nic or row[1] == nic:
                    return row[2]
        return None

    def _redhat_speed(self, nic):
        ethtool_cmd = 'ethtool %s' % nic
        ethtool_out = Popen(shlex.split(ethtool_cmd), stdout=PIPE).stdout.read().splitlines()
        _logger.debug('ethtool command output [%s]: %s', ethtool_cmd, ethtool_out)

        for line in ethtool_out:
            row = map(lambda s: s.strip(':'), line.strip().split())
            if len(row) >= 2:
                if row[0] == 'Speed':
                    if row[1].endswith('Mb/s'):
                        # convert to b/s
                        return str(int(row[1].replace('Mb/s', '')) * 1000 * 1000)
                    else:
                        return row[1]
        return None




"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    pyHai utility functions

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO

def _camel_case_to_underscore(name):
    """
    Take a camel cased name and returns a lower cased, underscore seperated name
    
    @param name: A name in CamelCase
    @type name: C{str}
    @return: An underscore seperated name
    @rtype: C{str}
    """
    if not name:
        raise ValueError('Can not create underscore seperated name from an empty string')

    if len(name) <= 1:
        return name.lower()

    result = StringIO()
    name_length = len(name)
    result.write(name[0].lower())
    for c in range(1, name_length):
        if name[c].isupper():
            result.write('_')
        result.write(name[c].lower())

    return result.getvalue()

def _underscore_to_camel_case(name):
    """
    Take a underscore seperated name and return a camel case name
    
    @param name: An underscore seperated name
    @type name: C{str}
    @return: A name in CamelCase
    @rtype: C{str}
    """
    if not name:
        raise ValueError('Can not CamelCase an empty string')
    if len(name) <= 1:
        return name.upper()
    return name.replace('_', ' ').title().replace(' ', '')



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A cross-platform Python auditor plugin

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import platform
import logging

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

class PythonPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):
        return {
            'python_build': platform.python_build(),
            'python_compiler': platform.python_compiler(),
            'python_branch': platform.python_branch(),
            'python_implementation': platform.python_implementation(),
            'python_revision': platform.python_revision(),
            'python_version': platform.python_version(),
            'python_version_tuple': platform.python_version_tuple(),
            }



"""
@copyright: 2011 Mark LaPerriere

@license:
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    U{http://www.apache.org/licenses/LICENSE-2.0}

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

@summary:
    A basic system information plugin that should work with most Windows systems

@author: Mark LaPerriere
@contact: pyhai@mindmind.com
@organization: Mind Squared Design / www.mindmind.com
@version: 0.1.0
@date: Sep 11, 2011
"""
from pyhai import AuditorPlugin
import platform
import logging

win32api = None

# set some default logging behavior
_logger = logging.getLogger(__name__)
_logger.addHandler(logging.NullHandler())

try:
    import win32api
except:
    _logger.exception('%s failed to import required Windows specific modules: %s', __name__, ', '.join('win32api'))
    raise

class SystemPlugin(AuditorPlugin):
    def run(self, *args, **kwargs):

        return {
            'computer_name': win32api.GetComputerName(),
            'domain': win32api.GetDomainName(),
            'uname': platform.uname(),
            'architecture': platform.architecture()[0],
            'platform': platform.platform(),
            'release': platform.release(),
            'system': platform.system(),
            'version': platform.version(),
            }
